@@ -139,12 +139,33 @@ sub as_string {
     }
 
     $result .= $self->year() . ", " if $self->year();
-    $result .= $self->OurPrice() . ", ";
+
+    $result .= $self->_best_effort_price() . ", ";
     $result .= $self->ASIN();
 
     return $result;
 }
 
+##################################################
+sub _best_effort_price {
+##################################################
+    my($self) = @_;
+
+    my $price;
+    if ($self->OurPrice()) {
+        $price = $self->OurPrice();
+    } elsif ($self->ThirdPartyNewPrice()) {
+        $price = $self->ThirdPartyNewPrice();
+    } elsif ($self->UsedPrice()) {
+        $price = $self->UsedPrice();
+    } else {
+        $price = '[$unknown]';
+    }
+
+    return $price;
+}
+
+
 ##################################################
 sub factory {
 ##################################################
@@ -416,8 +437,6 @@ Returns a list of ASINs of similar items for this item.
 Please check the subclasses of C<Net::Amazon::Property> for specialized 
 methods.
 
-=head1 SEE ALSO
-
 =head1 AUTHOR
 
 Mike Schilli, E<lt>m@perlmeister.comE<gt>
